package com.fr.plugin.jee.port;

import com.fr.base.*;
import com.fr.data.TableDataSource;
import com.fr.data.api.TableDataAssist;
import com.fr.decision.webservice.Response;
import com.fr.form.main.FormIO;
import com.fr.general.data.DataModel;
import com.fr.intelli.record.MetricRegistry;
import com.fr.io.TemplateWorkBookIO;
import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.main.TemplateWorkBook;
import com.fr.main.workbook.ResultWorkBook;
import com.fr.report.report.Report;
import com.fr.script.Calculator;
import com.fr.stable.Primitive;
import com.fr.stable.StringUtils;
import com.fr.stable.WriteActor;
import com.fr.stable.query.QueryFactory;
import com.fr.stable.query.condition.QueryCondition;
import com.fr.stable.query.data.DataList;
import com.fr.stable.query.restriction.RestrictionFactory;
import com.fr.web.utils.WebUtils;
import com.fr.write.main.WriteRWorkBook;
import com.tptj.plugin.hg.client.open.exception.CodeException;


import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author 秃破天际
 * @version 10.0
 * Created by 秃破天际 on 2020-09-07
 **/
public abstract class AbstractDataApi {



    protected List<Map<String,Object>> format(DataModel data )throws CodeException {
        List<Map<String,Object>> content = new ArrayList<Map<String,Object>>();
        try{
            int colCount = data.getColumnCount();
            String [] colNames = new String[colCount];
            for( int i=0; i<colCount; i++){
                colNames[i] = data.getColumnName(i);
            }
            int rowCount = data.getRowCount();
            for( int i=0;i<rowCount;i++ ){
                Map<String,Object> row = new HashMap<String,Object>();
                for( int j=0;j<colCount;j++ ){
                    row.put( colNames[j], format(data.getValueAt(i,j)) );
                }
                content.add(row);
            }
        }catch (Exception e){
            throw CodeException.create("10009",e.getMessage());
        }
        return content;
    }

    private final static SimpleDateFormat FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    private Object format(Object val)throws CodeException{
        if( val instanceof Date){
            return FORMAT.format((Date)val);
        }
        if( val instanceof Primitive){
            return null;
        }
        return val;
    }

    protected Calculator initCalculator(HttpServletRequest req, TableDataSource source ){
        Calculator calculator = Calculator.createCalculator();
        Map<String, Object> parameters = getSpParameters(req);
        ParameterMapNameSpace space = ParameterMapNameSpace.create(parameters);
        calculator.pushNameSpace( space );
        calculator.setAttribute( TableDataSource.KEY, source );
        return calculator;
    }

    protected TableDataSource getTableDataSource( String report )throws CodeException{
        try{
            if( report.endsWith(".frm") ){
                return FormIO.readForm(report);
            }
            return TemplateWorkBookIO.readTemplateWorkBook(report);
        }catch(Exception e){
            throw CodeException.create("10010",e.getMessage());
        }
    }

    protected String parseSql(HttpServletRequest req,String sql)throws CodeException{
        Parameter[] parameters = ParameterHelper.analyze4Parameters(sql, false);
        Map<String, Object> params = new HashMap<String, Object>();
        for( Parameter p :  parameters ){
            params.put(p.getName(),getSpParameters(req).get(p.getName()));
        }
        try{
            return TemplateUtils.render( sql, params,Calculator.createCalculator() );
        }catch(Exception e){
            throw CodeException.create("10011",e.getMessage());
        }
    }

    protected String[] getDataConfig( String id, String dsName )throws CodeException{
        Calculator cal = Calculator.createCalculator();
        Map<String,Object> parameters = new HashMap<String,Object>();
        parameters.put("id",id);
        ParameterMapNameSpace space = ParameterMapNameSpace.create(parameters);
        cal.pushNameSpace(space);
        DataModel rs = SynchronizedLiveDataModelUtils.getSELiveDataModel4Share(cal, dsName);
        if (rs == null) {
            TableData td = TableDataAssist.getTableData(cal, dsName);
            rs = td == null ? null : td.createDataModel(cal);
        }
        try{
            if( 0 == rs.getRowCount() ){
                throw new Exception("["+id+"] 对应的服务不存在！请联系管理员处理！");
            }
            if( 1 < rs.getRowCount() ){
                throw new Exception("["+id+"] 服务异常！一个ID只能对应一个数据服务！请联系管理员处理！");
            }
            int idxSql = rs.getColumnIndex("data");
            int connIdx = rs.getColumnIndex("source");
            return new String[]{
                    (String)rs.getValueAt(0,connIdx),(String)rs.getValueAt(0,idxSql)
            };
        }catch(Exception e){
            throw CodeException.create("10012",e.getMessage());
        }
    }

    protected ResultWorkBook getResultWorkBook( TemplateWorkBook workbook, HttpServletRequest req )throws CodeException {
        Map<String, Object> parameters = getSpParameters(req);
        return workbook.execute(parameters,new WriteActor());
    }

    protected Report getSheet(Map<String, Object> config, WriteRWorkBook resultBook ){
        //sheet名
        String name = (String) config.get("tag");
        if( StringUtils.isEmpty(name) ){
            name = resultBook.getReportName(0);
        }
        int idx = getSheetIndex( name, resultBook,0 );
        return resultBook.getReport(idx);
    }

    protected int getSheetIndex( String name, WriteRWorkBook resultBook, int dft){
        int count = resultBook.getReportCount();
        for( int i=0; i<count; i++ ){
            if( resultBook.getReportName(i).equals(name) ){
                return i;
            }
        }
        return dft;
    }

    /**
     * 获取计算报表需要的参数，在原产品的基础上，增加了body中JSON参数的读取
     * @param req
     * @return
     */
    protected Map<String,Object> getSpParameters(HttpServletRequest req){
        Map<String, Object> parameters = WebUtils.parameters4SessionIDInfor(req);
        if( null == parameters  ){
            parameters = new HashMap<String, Object>();
        }
        ServletInputStream in = null;
        try{
            int len = req.getContentLength();
            String charset = req.getCharacterEncoding();
            if( StringUtils.isEmpty(charset) ){
                charset = "UTF-8";
            }
            byte[] buffer = new byte[len];
            in = req.getInputStream();
            in.read(buffer, 0, len);
            String body = new String(buffer,charset);
            parameters.put("body",body);
            Map<String,Object> obj = new JSONObject(body).toMap();
            parameters.putAll(obj);
        }catch(Exception e){
            FineLoggerFactory.getLogger().info("####HUGH####:error={}",e.getMessage());
        }finally {
            if( null != in ){
                try{
                    in.close();
                }catch(Exception e){ }
            }
        }
        return parameters;
    }

    /**
     * 额外增加相应参数
     * @param req
     * @param defaultParameters
     * @param config
     * @return
     * @throws CodeException
     */
    public abstract Response mixCall(HttpServletRequest req, Map<String, Object> defaultParameters, Map<String, Object> config) throws CodeException ;

    /**
     * 获取要回显的其他内容
     * @param config
     * @return
     */
    public abstract Map<String, Object> getOptions(Map<String, Object> config);

    public Response call(HttpServletRequest req, Map<String, Object> defaultParameters, Map<String, Object> config) throws CodeException {
        Response res = mixCall(req, defaultParameters, config);
        Object data = res.getData();
        if( null == data || null == config ){
            return res;
        }
        config = getOptions(config);
        if( config.isEmpty() ){
            return res;
        }
        if( config.containsKey("url") ){
            String url = (String) config.get("url");
            if( !url.startsWith("http://") && !url.startsWith("https://") ){
                config.put("url",getBaseUrl(req)+config.get("url"));
            }
        }
        return Response.ok(data );
    }

    private String getBaseUrl( HttpServletRequest req ){
        String url = req.getScheme()+"://"+req.getServerName();
        int port = req.getServerPort();
        if( 443 != port && 80 != port ){
            url += ":"+ port;
        }
        url += req.getContextPath()+req.getServletPath();
        return url;
    }

}
